package org.orbroker
import org.orbroker.callback.ExecutionCallback
import java.sql.{ SQLWarning, SQLException, Connection }

private[orbroker] trait JdbcType[-T] {
  def close(t: T)
  def getWarnings(t: T): SQLWarning
}

private[orbroker] object JdbcCloser {
  implicit object JdbcConnection extends JdbcType[java.sql.Connection] {
    def close(conn: java.sql.Connection) = conn.close()
    def getWarnings(conn: java.sql.Connection) = conn.getWarnings
  }
  implicit object JdbcResultSet extends JdbcType[java.sql.ResultSet] {
    def close(rs: java.sql.ResultSet) = rs.close()
    def getWarnings(rs: java.sql.ResultSet) = rs.getWarnings
  }
  implicit object JdbcStatement extends JdbcType[java.sql.Statement] {
    def close(ps: java.sql.Statement) = ps.close()
    def getWarnings(ps: java.sql.Statement) = ps.getWarnings
  }

  implicit def toCloseable[T](candidate: T)(implicit exe: JdbcType[T], callback: ExecutionCallback) = new {
    def checkAndClose(stm: Symbol = null) = exe match {
      case JdbcConnection => checkWarningsAndClose(candidate, callback, exe, Right(candidate.asInstanceOf[java.sql.Connection]))
      case _ => checkWarningsAndClose(candidate, callback, exe, Left(stm))
    }
  }

  private def checkWarningsAndClose[T](jdbc: T, callback: ExecutionCallback, exe: JdbcType[T], src: Either[Symbol, Connection]) {
    try {
      handleWarning(src, callback, exe.getWarnings(jdbc))
    } catch {
      case e: SQLException => // Ignore
    }
    finally {
      exe.close(jdbc)
    }
  }

  private def handleWarning(src: Either[Symbol, Connection], callback: ExecutionCallback, warning: java.sql.SQLWarning) {
    if (warning != null) {
      callback.onWarning(warning, src)
      handleWarning(src, callback, warning.getNextWarning)
    }
  }

}