package org.orbroker.callback

import java.util.logging._

/**
 * Callback for logging to the JDK logger.
 * @param logger The JDK logger
 * @param sqlLevel The log level for executed SQL statements
 * @param thresholdMillis Only log statements taking longer than threshold
 * @param ignoreWarnings Set of SQL state codes to ignore when warned
 */
class JDKLoggingCallback(logger: Logger, sqlLevel: Level, thresholdMillis: Int = 0,
                         ignoreWarnings: Set[String] = Set.empty) extends ExecutionCallback with LogEmbeddedValues {
  def onSQL(id: Symbol, sql: String, parms: Seq[Any], executionTime: Int) {
    val millis = executionTime / 1000
    if (millis >= thresholdMillis) {
      val sqlWithValues = embedValues(sql.trim, parms)
      logger.log(sqlLevel,
        "Executed '%s' (%,d ms): %s".format(
          id.name, int2Integer(millis), sqlWithValues))
    }
  }

  def onSQLBatch(id: Symbol, sql: String, parms: Seq[Seq[Any]], executionTime: Int) {
    val millis = executionTime / 1000
    if (millis >= thresholdMillis) {
      logger.log(sqlLevel,
        "Executed '%s' %,d times (%,d ms): %s".format(
          id.name, int2Integer(parms.size), int2Integer(millis), sql.trim))
    }
  }

  def onWarning(warning: java.sql.SQLWarning, source: Either[Symbol, java.sql.Connection]) = {
    if (!ignoreWarnings.contains(warning.getSQLState)) {
      source match {
        case Left(id) ⇒ logger.warning("Statement '%s': [%s] %s".format(id.name, warning.getSQLState, warning.getMessage))
        case Right(conn) ⇒ logger.warning("%s: [%s] %s".format(conn.getClass.getName, warning.getSQLState, warning.getMessage))
      }
    }
  }

}
